#include <File_Archiver.h>
#include <filesystem>

using namespace inaf::oasbo::Archivers;

FileArchiver::FileArchiver(std::string parent, std::string dest) :
		_open(false), parentPath(parent){
	this->dest = std::string(parentPath).append("/").append(dest).append(".raw");
}

FileArchiver::FileArchiver(std::string dest) :
		FileArchiver(std::string(std::getenv("HOME")).append("/ArchiveRAW"),
				dest) {
}

FileArchiver::FileArchiver() :
		FileArchiver(std::string(std::getenv("HOME")).append("/ArchiveRAW"),
				".tmp") {
}

int FileArchiver::write(PacketLib::BasePacket &pack) {
	if (myfile == nullptr || !myfile->is_open()) {
		std::cerr << "Archiver err: File not opened" << std::endl;
		return -1;
	}
	myfile->write((char*) pack.getBinaryPointer(),
			pack.getHeaderSize() + pack.getPayloadSize() + pack.getTailSize());
	return 1;
}

int FileArchiver::write(PacketLib::BasePacket &pack, std::string dest) {
	namespace fs = std::filesystem;
	fs::path dest_path(dest);
	if (!fs::exists(dest_path.parent_path())
			|| std::filesystem::exists(dest_path)) {
		this->dest = computeFileName(dest);
	}
	std::ofstream *file = new std::ofstream(dest, std::ios::binary);
	file->write((char*) pack.getBinaryPointer(),
			pack.getHeaderSize() + pack.getPayloadSize() + pack.getTailSize());
	file->close();
	delete file;
	return 1;
}

int FileArchiver::open() {
	namespace fs = std::filesystem;
	fs::path dest_path(this->dest);

	if (!fs::exists(dest_path.parent_path())
			|| std::filesystem::exists(dest_path)) {
		this->dest = computeFileName(this->dest);
	}
	if (myfile != nullptr)
		delete myfile;
	this->myfile = new std::ofstream(this->dest, std::ios::binary);
	if(myfile->good()){
		_open = true;
	}
	return _open;
}

int FileArchiver::close() {
	_open = false;
	if (myfile != nullptr && myfile->is_open())
		myfile->close();
	if (myfile != nullptr)
		delete myfile;
	myfile = nullptr;
	return 1;
}


bool FileArchiver::is_open() {
	return _open;
}

std::string FileArchiver::getDest() {
	return dest;
}

void FileArchiver::setDest(std::string dest) {
	this->dest = std::string(this->parentPath).append(dest).append(".raw");
}

std::string FileArchiver::computeFileName(std::string fileName) {
	namespace fs = std::filesystem;
// resolve env var if present
	if(fileName.at(0) == '$'){
	    size_t pos = fileName.find('/'); // find first '/'
        std::string env_var_name = fileName.substr(1, pos - 1);
        char* env_var_value = std::getenv(env_var_name.c_str());
        std::string env_var;
        if (env_var_value != nullptr)
            env_var = std::string(std::getenv(env_var_name.c_str()));
        else // env var does not exist! Replace it with $HOME
            env_var = std::string(std::getenv("HOME"));

        fileName = std::string(env_var).append(fileName.substr(pos));
	}
// Check if the directory of the file path exists
	fs::path dest_path(fileName);
	if (!fs::exists(dest_path.parent_path())) {
		// Create the directory and any missing parent directories
		fs::create_directories(dest_path.parent_path());
		return dest_path.string();
	}
// Check if the the file already exists and append an incremental suffix
	if (fs::exists(dest_path)) {
		std::string extension = dest_path.extension().string();
		dest_path.replace_extension("");
		int count = 0;
		while (fs::exists(
				dest_path.string() + "_(" + std::to_string(++count) + ")"
						+ extension))
			;
		dest_path += "_(" + std::to_string(count) + ")" + extension;
	}
	return dest_path.string();
}

void FileArchiver::updateDest() {

}
